@@ -1,13 +1,16 @@
 package Mojo::SQLite::PubSub;
 use Mojo::Base 'Mojo::EventEmitter';
 
+use Mojo::JSON qw(from_json to_json);
 use Scalar::Util 'weaken';
 
-our $VERSION = '0.021';
+our $VERSION = '0.022';
 
 has 'poll_interval';
 has 'sqlite';
 
+sub json { ++$_[0]{json}{$_[1]} and return $_[0] }
+
 sub listen {
   my ($self, $name, $cb) = @_;
   $self->_db->listen($name) unless @{$self->{chans}{$name} ||= []};
@@ -15,7 +18,7 @@ sub listen {
   return $cb;
 }
 
-sub notify { $_[0]->_db->notify(@_[1, 2]) and return $_[0] }
+sub notify { $_[0]->_db->notify(_json(@_)) and return $_[0] }
 
 sub unlisten {
   my ($self, $name, $cb) = @_;
@@ -34,14 +37,14 @@ sub _db {
 
   return $self->{db} if $self->{db};
 
-  my %options;
-  $options{notification_poll_interval} = $self->poll_interval if defined $self->poll_interval;
-  my $db = $self->{db} = $self->sqlite->db(%options);
+  my $db = $self->{db} = $self->sqlite->db;
+  $db->notification_poll_interval($self->poll_interval) if defined $self->poll_interval;
   weaken $db->{sqlite};
   weaken $self;
   $db->on(
     notification => sub {
       my ($db, $name, $payload) = @_;
+      $payload = eval { from_json $payload } if $self->{json}{$name};
       for my $cb (@{$self->{chans}{$name}}) { $self->$cb($payload) }
     }
   );
@@ -58,8 +61,12 @@ sub _db {
   return $db;
 }
 
+sub _json { $_[1], $_[0]{json}{$_[1]} ? to_json $_[2] : $_[2] }
+
 1;
 
+=encoding utf8
+
 =head1 NAME
 
 Mojo::SQLite::PubSub - Publish/Subscribe
@@ -73,7 +80,7 @@ Mojo::SQLite::PubSub - Publish/Subscribe
     my ($pubsub, $payload) = @_;
     say "Received: $payload";
   });
-  $pubsub->notify(foo => 'bar');
+  $pubsub->notify(foo => 'I ♥ SQLite!');
   $pubsub->unlisten(foo => $cb);
 
 =head1 DESCRIPTION
@@ -125,12 +132,27 @@ L<Mojo::SQLite> object this publish/subscribe container belongs to.
 L<Mojo::SQLite::PubSub> inherits all methods from L<Mojo::EventEmitter> and
 implements the following new ones.
 
+=head2 json
+
+  $pubsub = $pubsub->json('foo');
+
+Activate automatic JSON encoding and decoding with L<Mojo::JSON/"to_json"> and
+L<Mojo::JSON/"from_json"> for a channel.
+
+  # Send and receive data structures
+  $pubsub->json('foo')->listen(foo => sub {
+    my ($pubsub, $payload) = @_;
+    say $payload->{bar};
+  });
+  $pubsub->notify(foo => {bar => 'I ♥ SQLite!'});
+
 =head2 listen
 
   my $cb = $pubsub->listen(foo => sub {...});
 
 Subscribe to a channel, there is no limit on how many subscribers a channel can
-have.
+have. Automatic decoding of JSON text to Perl values can be activated with
+L</"json">.
 
   # Subscribe to the same channel twice
   $pubsub->listen(foo => sub {
@@ -145,9 +167,11 @@ have.
 =head2 notify
 
   $pubsub = $pubsub->notify('foo');
-  $pubsub = $pubsub->notify(foo => 'bar');
+  $pubsub = $pubsub->notify(foo => 'I ♥ SQLite!');
+  $pubsub = $pubsub->notify(foo => {bar => 'baz'});
 
-Notify a channel.
+Notify a channel. Automatic encoding of Perl values to JSON text can be
+activated with L</"json">.
 
 =head2 unlisten
 
