@@ -5,7 +5,7 @@ use Mojo::Collection;
 use Mojo::JSON 'from_json';
 use Mojo::Util 'tablify';
 
-our $VERSION = '0.021';
+our $VERSION = '0.022';
 
 has 'sth';
 
@@ -51,21 +51,20 @@ sub text { tablify shift->arrays }
 sub _collect { Mojo::Collection->new(@_) }
 
 sub _expand {
-  my ($self, @data) = @_;
+  my ($self, @rows) = @_;
   
-  return @data unless $data[0] and $self->{expand};
+  return @rows unless $self->{expand} and $rows[0];
   
-  for my $data (@data) {
-    if (ref $data eq 'HASH') {
-      $data->{$_} = from_json $data->{$_}
-        for grep { $data->{$_} and $self->{expand}{json}{$_} } keys %$data;
-    } else {
-      $data->[$_] = from_json $data->[$_]
-        for grep { $data->[$_] and $self->{expand}{json}{$self->columns->[$_]} } 0..$#$data;
-    }
+  if (ref $rows[0] eq 'HASH') {
+    my @json_names = keys %{$self->{expand}{json}};
+    for my $r (@rows) { $r->{$_} = from_json $r->{$_} for grep { $r->{$_} } @json_names }
+  } else {
+    my $cols = $self->columns;
+    my @json_idxs = grep { $self->{expand}{json}{$cols->[$_]} } 0..$#$cols;
+    for my $r (@rows) { $r->[$_] = from_json $r->[$_] for grep { $r->[$_] } @json_idxs }
   }
   
-  return @data;
+  return @rows;
 }
 
 1;
@@ -146,10 +145,10 @@ Return column names as an array reference.
   $results = $results->expand(json => 'some_json');
   $results = $results->expand(json => ['some_json','other_json']);
 
-Decode specified fields from a particular format for all rows. Currently only
-the C<json> text format is recognized. The names must exactly match the column
-names as returned by L</"columns">; it is recommended to use explicit aliases
-in the query for consistent column names.
+Decode specified fields from a particular format to Perl values for all rows.
+Currently only the C<json> text format is recognized. The names must exactly
+match the column names as returned by L</"columns">; it is recommended to use
+explicit aliases in the query for consistent column names.
 
   # Expand JSON
   $results->expand(json => 'json_field')->hashes->map(sub { $_->{foo}{bar} })->join("\n")->say;
